@@ -21,7 +21,6 @@
 #ifndef vte_debug_h_included
 #define vte_debug_h_included
 
-#ident "$Id: debug.h,v 1.15 2003/05/05 23:23:46 nalin Exp $"
 
 #include <glib.h>
 
@@ -40,11 +39,13 @@ typedef enum {
 	VTE_DEBUG_PTY		= 1 << 9,
 	VTE_DEBUG_CURSOR	= 1 << 10,
 	VTE_DEBUG_KEYBOARD	= 1 << 11,
-	VTE_DEBUG_LIFECYCLE	= 1 << 12
+	VTE_DEBUG_LIFECYCLE	= 1 << 12,
+	VTE_DEBUG_TRIE		= 1 << 13,
+	VTE_DEBUG_WORK	= 1 << 14
 } VteDebugFlags;
 
 void _vte_debug_parse_string(const char *string);
-gboolean _vte_debug_on(VteDebugFlags flags);
+gboolean _vte_debug_on(VteDebugFlags flags) G_GNUC_CONST;
 
 G_END_DECLS
 
