@@ -19,7 +19,6 @@
 
 #include "../config.h"
 #include <sys/types.h>
-#include <stdio.h>
 #include <string.h>
 #include <glib-object.h>
 #include "debug.h"
@@ -29,49 +28,42 @@
 #include "trie.h"
 
 struct _vte_matcher {
-	gboolean initialized;
-	enum {
-		_vte_matcher_table,
-		_vte_matcher_trie
-	} type;
-	struct _vte_table *table;
-	struct _vte_trie *trie;
+	_vte_matcher_match_func match; /* shortcut to the most common op */
+	struct _vte_matcher_impl *impl;
 };
 
 static GStaticMutex _vte_matcher_mutex = G_STATIC_MUTEX_INIT;
 static GCache *_vte_matcher_cache = NULL;
+static struct _vte_matcher_impl dummy_vte_matcher_trie = {
+	&_vte_matcher_trie
+};
+static struct _vte_matcher_impl dummy_vte_matcher_table = {
+	&_vte_matcher_table
+};
 
 /* Add a string to the matcher. */
 static void
-_vte_matcher_add(struct _vte_matcher *matcher,
+_vte_matcher_add(const struct _vte_matcher *matcher,
 		 const char *pattern, gssize length,
 		 const char *result, GQuark quark)
 {
-	switch (matcher->type) {
-	case _vte_matcher_table:
-		_vte_table_add(matcher->table, pattern, length, result, quark);
-		break;
-	case _vte_matcher_trie:
-		_vte_trie_add(matcher->trie, pattern, length, result, quark);
-		break;
-	}
+	matcher->impl->klass->add(matcher->impl, pattern, length, result, quark);
 }
 
 /* Loads all sequences into matcher */
 static void
-_vte_matcher_init(struct _vte_matcher *matcher, char *emulation,
+_vte_matcher_init(struct _vte_matcher *matcher, const char *emulation,
 		  struct _vte_termcap *termcap)
 {
 	const char *code, *value;
 	gboolean found_cr = FALSE, found_lf = FALSE;
 	gssize stripped_length;
 	char *stripped;
-	char *tmp;
 	int i;
 
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_LIFECYCLE)) {
-		fprintf(stderr, "_vte_matcher_init()\n");
+		g_printerr("_vte_matcher_init()\n");
 	}
 #endif
 
@@ -84,9 +76,11 @@ _vte_matcher_init(struct _vte_matcher *matcher, char *emulation,
 			continue;
 		}
 		code = _vte_terminal_capability_strings[i].capability;
-		tmp = _vte_termcap_find_string(termcap, emulation, code);
-		if ((tmp != NULL) && (tmp[0] != '\0')) {
-			_vte_termcap_strip(tmp, &stripped, &stripped_length);
+		stripped = _vte_termcap_find_string_length(termcap,
+                                                           emulation,
+                                                           code,
+                                                           &stripped_length);
+		if (stripped[0] != '\0') {
 			_vte_matcher_add(matcher, stripped, stripped_length,
 					 code, 0);
 			if (stripped[0] == '\r') {
@@ -98,9 +92,8 @@ _vte_matcher_init(struct _vte_matcher *matcher, char *emulation,
 					found_lf = TRUE;
 				}
 			}
-			g_free(stripped);
 		}
-		g_free(tmp);
+		g_free(stripped);
 	}
 
 	/* Add emulator-specific sequences. */
@@ -111,10 +104,8 @@ _vte_matcher_init(struct _vte_matcher *matcher, char *emulation,
 		     i++) {
 			code = _vte_xterm_capability_strings[i].code;
 			value = _vte_xterm_capability_strings[i].value;
-			_vte_termcap_strip(code, &stripped, &stripped_length);
-			_vte_matcher_add(matcher, stripped, stripped_length,
+			_vte_matcher_add(matcher, code, strlen (code),
 					 value, 0);
-			g_free(stripped);
 		}
 	}
 
@@ -128,9 +119,9 @@ _vte_matcher_init(struct _vte_matcher *matcher, char *emulation,
 
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_TRIE)) {
-		fprintf(stderr, "Trie contents:\n");
+		g_printerr("Trie contents:\n");
 		_vte_matcher_print(matcher);
-		fprintf(stderr, "\n");
+		g_printerr("\n");
 	}
 #endif
 }
@@ -144,29 +135,18 @@ _vte_matcher_create(gpointer key)
 
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_LIFECYCLE)) {
-		fprintf(stderr, "_vte_matcher_create()\n");
+		g_printerr("_vte_matcher_create()\n");
 	}
 #endif
 	ret = g_slice_new(struct _vte_matcher);
-	ret->initialized = FALSE;
-	ret->type = _vte_matcher_trie;
-	ret->table = NULL;
-	ret->trie = NULL;
+	ret->impl = &dummy_vte_matcher_trie;
+	ret->match = NULL;
 
 	if (strcmp(emulation, "xterm") == 0) {
-		ret->type = _vte_matcher_table;
+		ret->impl = &dummy_vte_matcher_table;
 	} else
 	if (strcmp(emulation, "dtterm") == 0) {
-		ret->type = _vte_matcher_table;
-	}
-
-	switch (ret->type) {
-	case _vte_matcher_table:
-		ret->table = _vte_table_new();
-		break;
-	case _vte_matcher_trie:
-		ret->trie = _vte_trie_new();
-		break;
+		ret->impl = &dummy_vte_matcher_table;
 	}
 
 	return ret;
@@ -180,21 +160,17 @@ _vte_matcher_destroy(gpointer value)
 
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_LIFECYCLE)) {
-		fprintf(stderr, "_vte_matcher_destroy()\n");
+		g_printerr("_vte_matcher_destroy()\n");
 	}
 #endif
-	if (matcher->table != NULL) {
-		_vte_table_free(matcher->table);
-	}
-	if (matcher->trie != NULL) {
-		_vte_trie_free(matcher->trie);
-	}
+	if (matcher->match != NULL) /* do not call destroy on dummy values */
+		matcher->impl->klass->destroy(matcher->impl);
 	g_slice_free(struct _vte_matcher, matcher);
 }
 
 /* Create and init matcher. */
 struct _vte_matcher *
-_vte_matcher_new(char *emulation, struct _vte_termcap *termcap)
+_vte_matcher_new(const char *emulation, struct _vte_termcap *termcap)
 {
 	struct _vte_matcher *ret = NULL;
 	g_static_mutex_lock(&_vte_matcher_mutex);
@@ -205,15 +181,17 @@ _vte_matcher_new(char *emulation, struct _vte_termcap *termcap)
 
 	if (_vte_matcher_cache == NULL) {
 		_vte_matcher_cache = g_cache_new(_vte_matcher_create,
-				_vte_matcher_destroy, g_strdup, g_free,
+				_vte_matcher_destroy,
+			       	(GCacheDupFunc) g_strdup, g_free,
 				g_str_hash, g_direct_hash, g_str_equal);
 	}
 
-	ret = g_cache_insert(_vte_matcher_cache, emulation);
+	ret = g_cache_insert(_vte_matcher_cache, (gpointer) emulation);
 
-	if (!ret->initialized) {
+	if (ret->match == NULL) {
+		ret->impl = ret->impl->klass->create();
+		ret->match = ret->impl->klass->match;
 		_vte_matcher_init(ret, emulation, termcap);
-		ret->initialized = TRUE;
 	}
 
 	g_static_mutex_unlock(&_vte_matcher_mutex);
@@ -237,31 +215,15 @@ _vte_matcher_match(struct _vte_matcher *matcher,
 		   const char **res, const gunichar **consumed,
 		   GQuark *quark, GValueArray **array)
 {
-	switch (matcher->type) {
-	case _vte_matcher_table:
-		return _vte_table_match(matcher->table, pattern, length,
+	return matcher->match(matcher->impl, pattern, length,
 					res, consumed, quark, array);
-		break;
-	case _vte_matcher_trie:
-		return _vte_trie_match(matcher->trie, pattern, length,
-				       res, consumed, quark, array);
-		break;
-	}
-	return NULL;
 }
 
 /* Dump out the contents of a matcher, mainly for debugging. */
 void
 _vte_matcher_print(struct _vte_matcher *matcher)
 {
-	switch (matcher->type) {
-	case _vte_matcher_table:
-		_vte_table_print(matcher->table);
-		break;
-	case _vte_matcher_trie:
-		_vte_trie_print(matcher->trie);
-		break;
-	}
+	matcher->impl->klass->print(matcher->impl);
 }
 
 /* Free a parameter array.  Most of the GValue elements can clean up after
@@ -272,15 +234,11 @@ _vte_matcher_free_params_array(GValueArray *params)
 {
 	guint i;
 	GValue *value;
-	gpointer ptr;
 	if (params != NULL) {
 		for (i = 0; i < params->n_values; i++) {
 			value = g_value_array_get_nth(params, i);
 			if (G_VALUE_HOLDS_POINTER(value)) {
-				ptr = g_value_get_pointer(value);
-				if (ptr != NULL) {
-					g_free(ptr);
-				}
+				g_free(g_value_get_pointer(value));
 				g_value_set_pointer(value, NULL);
 			}
 		}
