@@ -21,7 +21,6 @@
 #ifndef vte_ring_h_included
 #define vte_ring_h_included
 
-#ident "$Id: ring.h,v 1.12 2002/09/11 06:10:46 nalin Exp $"
 
 #include <glib.h>
 
@@ -44,6 +43,7 @@ struct _VteRing {
 #define _vte_ring_length(__ring) ((__ring)->length)
 #define _vte_ring_next(__ring) ((__ring)->delta + (__ring)->length)
 #define _vte_ring_max(__ring) ((__ring)->max)
+#ifdef VTE_DEBUG
 #define _vte_ring_at(__ring, __position) \
 	((__ring)->array[__position % (__ring)->max] ? \
 	 (__ring)->array[__position % (__ring)->max] : \
@@ -53,6 +53,10 @@ struct _VteRing {
 		  (__ring)->delta, (__ring)->length, (__ring)->max, \
 		  (__ring)->delta + (__ring)->length, \
 		  __LINE__), (gpointer) NULL))
+#else
+#define _vte_ring_at(__ring, __position) \
+	((__ring)->array[__position % (__ring)->max])
+#endif
 #define _vte_ring_index(__ring, __cast, __position) \
 	(__cast) _vte_ring_at(__ring, __position)
 
@@ -62,6 +66,7 @@ VteRing *_vte_ring_new(glong max_elements,
 VteRing *_vte_ring_new_with_delta(glong max_elements, glong delta,
 				  VteRingFreeFunc free, gpointer data);
 void _vte_ring_insert(VteRing *ring, glong position, gpointer data);
+void _vte_ring_insert_preserve(VteRing *ring, glong position, gpointer data);
 void _vte_ring_remove(VteRing *ring, glong position, gboolean free_element);
 void _vte_ring_append(VteRing *ring, gpointer data);
 void _vte_ring_free(VteRing *ring, gboolean free_elements);
