@@ -18,7 +18,6 @@
 
 /* The interfaces in this file are subject to change at any time. */
 
-#ident "$Id: vtedraw.c,v 1.16 2004/04/20 05:16:56 nalin Exp $"
 
 #include "../config.h"
 
@@ -35,69 +34,106 @@
 #include "vteskel.h"
 #include "vtexft.h"
 
-struct _vte_draw_impl
+static const struct _vte_draw_impl
 *_vte_draw_impls[] = {
-	&_vte_draw_skel,
 #ifndef X_DISPLAY_MISSING
 #ifdef HAVE_XFT2
 	&_vte_draw_xft,
-#endif
-#endif
+#endif /* HAVE_XFT2 */
+#endif /* !X_DISPLAY_MISSING */
 	&_vte_draw_ft2,
-#if GTK_CHECK_VERSION(2,2,0)
 #ifndef X_DISPLAY_MISSING
 #ifdef HAVE_GL
 	&_vte_draw_gl,
-#endif
-#endif
-#endif
+#endif /* HAVE_GL */
+#endif /* !X_DISPLAY_MISSING */
 	&_vte_draw_pango,
 #ifndef X_DISPLAY_MISSING
 #ifdef HAVE_PANGOX
 	&_vte_draw_pango_x,
-#endif
-#endif
+#endif /* HAVE_PANGOX */
+#endif /* !X_DISPLAY_MISSING */
 };
 
+static gboolean
+_vte_draw_init_user (struct _vte_draw *draw)
+{
+	const gchar *env;
+	gchar **strv, **s;
+	guint i;
+	gboolean success = TRUE;
+
+	env = g_getenv("VTE_BACKEND");
+	if (!env) {
+		return FALSE;
+	}
+
+	strv = g_strsplit(env, ":;, \t", -1);
+	for (s = strv; *s; s++) {
+		if (g_ascii_strcasecmp(*s, _vte_draw_skel.name) == 0) {
+			draw->impl = &_vte_draw_skel;
+			goto out;
+		}
+		for (i = 0; i < G_N_ELEMENTS(_vte_draw_impls); i++) {
+			if (g_ascii_strcasecmp(*s, _vte_draw_impls[i]->name) == 0) {
+				if (_vte_draw_impls[i]->check(draw, draw->widget)) {
+					draw->impl = _vte_draw_impls[i];
+					goto out;
+				}
+			}
+		}
+	}
+
+	success = FALSE;
+out:
+	g_strfreev(strv);
+	return success;
+}
+
+
+static gboolean
+_vte_draw_init_default (struct _vte_draw *draw)
+{
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS(_vte_draw_impls); i++) {
+		if (_vte_draw_impls[i]->check(draw, draw->widget)) {
+			draw->impl = _vte_draw_impls[i];
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
 struct _vte_draw *
 _vte_draw_new(GtkWidget *widget)
 {
-	int i;
 	struct _vte_draw *draw;
-	char *var;
 
 	/* Create the structure. */
-	draw = g_malloc0(sizeof(struct _vte_draw));
-	g_object_ref(G_OBJECT(widget));
-	draw->widget = widget;
+	draw = g_slice_new0(struct _vte_draw);
+	draw->widget = g_object_ref(widget);
 	draw->started = FALSE;
 
-	/* Let each implementation decide if it's right for the job. */
-	for (i = 0; i < G_N_ELEMENTS(_vte_draw_impls); i++) {
-		if (_vte_draw_impls[i]->environment != NULL) {
-			var = getenv(_vte_draw_impls[i]->environment);
-			if (var != NULL) {
-				if (atol(var) == 0) {
-					continue;
-				}
-			}
-		}
-		if (_vte_draw_impls[i]->check(draw, draw->widget)) {
-			draw->impl = _vte_draw_impls[i];
-			draw->impl->create(draw, draw->widget);
-			break;
-		}
+	/* Allow the user to specify her preferred backends */
+	if (!_vte_draw_init_user (draw) &&
+			/* Otherwise use the first thing that works */
+			!_vte_draw_init_default (draw)) {
+		/* Something has to work. */
+		g_assert_not_reached ();
+		draw->impl = &_vte_draw_skel;
 	}
 
-	/* Something has to have worked. */
-	g_assert(i < G_N_ELEMENTS(_vte_draw_impls));
-
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_MISC)) {
-		fprintf(stderr, "Using %s.\n", draw->impl->name);
+		g_printerr("Using %s.\n", draw->impl->name);
 	}
 #endif
 
+	draw->impl->create(draw, draw->widget);
+
 	return draw;
 }
 
@@ -105,16 +141,12 @@ void
 _vte_draw_free(struct _vte_draw *draw)
 {
 	draw->impl->destroy(draw);
-	draw->impl = NULL;
-	draw->impl_data = NULL;
 
-	if (GTK_IS_WIDGET(draw->widget)) {
-		g_object_unref(G_OBJECT(draw->widget));
-		draw->widget = NULL;
+	if (draw->widget != NULL) {
+		g_object_unref(draw->widget);
 	}
-	draw->started = FALSE;
 
-	g_free(draw);
+	g_slice_free(struct _vte_draw, draw);
 }
 
 GdkVisual *
@@ -129,9 +161,7 @@ GdkColormap *
 _vte_draw_get_colormap(struct _vte_draw *draw, gboolean maybe_use_default)
 {
 	GdkColormap *colormap;
-#if GTK_CHECK_VERSION(2,2,0)
 	GdkScreen *screen;
-#endif
 	g_return_val_if_fail(draw->impl != NULL, NULL);
 	g_return_val_if_fail(draw->impl->get_colormap != NULL, NULL);
 	colormap = draw->impl->get_colormap(draw);
@@ -141,16 +171,12 @@ _vte_draw_get_colormap(struct _vte_draw *draw, gboolean maybe_use_default)
 	if (!maybe_use_default) {
 		return NULL;
 	}
-#if GTK_CHECK_VERSION(2,2,0)
 	if (gtk_widget_has_screen(draw->widget)) {
 		screen = gtk_widget_get_screen(draw->widget);
 	} else {
 		screen = gdk_display_get_default_screen(gtk_widget_get_display(draw->widget));
 	}
 	colormap = gdk_screen_get_default_colormap(screen);
-#else
-	colormap = gdk_colormap_get_system();
-#endif
 	return colormap;
 }
 
@@ -160,7 +186,7 @@ _vte_draw_start(struct _vte_draw *draw)
 	g_return_if_fail(GTK_WIDGET_REALIZED(draw->widget));
 	g_return_if_fail(draw->impl != NULL);
 	g_return_if_fail(draw->impl->start != NULL);
-	g_object_ref(G_OBJECT(draw->widget->window));
+	g_object_ref(draw->widget->window);
 	draw->impl->start(draw);
 	draw->started = TRUE;
 }
@@ -172,16 +198,18 @@ _vte_draw_end(struct _vte_draw *draw)
 	g_return_if_fail(draw->impl != NULL);
 	g_return_if_fail(draw->impl->end != NULL);
 	draw->impl->end(draw);
-	g_object_unref(G_OBJECT(draw->widget->window));
+	g_object_unref(draw->widget->window);
 	draw->started = FALSE;
 }
 
 void
-_vte_draw_set_background_color(struct _vte_draw *draw, GdkColor *color)
+_vte_draw_set_background_color(struct _vte_draw *draw,
+			       GdkColor *color,
+			       guint16 opacity)
 {
 	g_return_if_fail(draw->impl != NULL);
 	g_return_if_fail(draw->impl->set_background_color != NULL);
-	draw->impl->set_background_color(draw, color);
+	draw->impl->set_background_color(draw, color, opacity);
 }
 
 void
