@@ -38,15 +38,22 @@
 
 #define FONT_INDEX_FUDGE 10
 #define CHAR_WIDTH_FUDGE 10
-#define INVALID_GLYPH    -1
+#define INVALID_GLYPH    GINT_TO_POINTER(-1)
 
 static FT_Face _vte_glyph_cache_face_for_char(struct _vte_glyph_cache *cache,
 					      gunichar c);
 
-static int
-_vte_direct_compare(gconstpointer a, gconstpointer b)
+void
+_vte_glyph_free(struct _vte_glyph *glyph)
 {
-	return GPOINTER_TO_INT(a) - GPOINTER_TO_INT(b);
+	g_free(glyph);
+}
+
+static void
+_vte_cached_glyph_free(gpointer glyph)
+{
+	if (glyph != INVALID_GLYPH)
+		_vte_glyph_free(glyph);
 }
 
 struct _vte_glyph_cache *
@@ -57,9 +64,9 @@ _vte_glyph_cache_new(void)
 
 	ret = g_slice_new(struct _vte_glyph_cache);
 
-	ret->patterns = g_array_new(TRUE, TRUE, sizeof(FcPattern*));
+	ret->patterns = g_ptr_array_new();
 	ret->faces = NULL;
-	ret->cache = g_tree_new(_vte_direct_compare);
+	ret->cache = g_hash_table_new_full(NULL, NULL, NULL, _vte_cached_glyph_free);
 	ret->ft_load_flags = 0;
 	ret->ft_render_flags = 0;
 	ret->width = 0;
@@ -72,25 +79,10 @@ _vte_glyph_cache_new(void)
 	return ret;
 }
 
-void
-_vte_glyph_free(struct _vte_glyph *glyph)
-{
-	g_free(glyph);
-}
-
-static gboolean
-free_tree_value(gpointer key, gpointer value, gpointer data)
-{
-	if (GPOINTER_TO_INT(value) != INVALID_GLYPH) {
-		_vte_glyph_free(value);
-	}
-	return FALSE;
-}
 
 void
 _vte_glyph_cache_free(struct _vte_glyph_cache *cache)
 {
-	GList *iter;
 	int i;
 
 	g_return_if_fail(cache != NULL);
@@ -98,37 +90,24 @@ _vte_glyph_cache_free(struct _vte_glyph_cache *cache)
 	/* Destroy the patterns. */
 	if (cache->patterns != NULL) {
 		for (i = 0; i < cache->patterns->len; i++) {
-			FcPatternDestroy(g_array_index(cache->patterns,
-						       FcPattern*,
-						       i));
+			FcPatternDestroy(g_ptr_array_index(cache->patterns, i));
 		}
-		g_array_free(cache->patterns, TRUE);
-		cache->patterns = NULL;
+		g_ptr_array_free(cache->patterns, TRUE);
 	}
 
 	/* Close all faces. */
-	for (iter = cache->faces; iter != NULL; iter = g_list_next(iter)) {
-		FT_Done_Face((FT_Face) iter->data);
-		iter->data = NULL;
-	}
-	cache->faces = NULL;
+	g_list_foreach(cache->faces, (GFunc)FT_Done_Face, NULL);
+	g_list_free(cache->faces);
 
 	/* Free the glyph tree. */
-	g_tree_foreach(cache->cache, free_tree_value, NULL);
-	cache->cache = NULL;
+	g_hash_table_destroy(cache->cache);
 
 	/* Close the FT library. */
 	if (cache->ft_library) {
 		FT_Done_FreeType(cache->ft_library);
-		cache->ft_library = NULL;
 	}
 
 	/* Free the cache. */
-	cache->ft_load_flags = 0;
-	cache->ft_render_flags = 0;
-	cache->width = 0;
-	cache->height = 0;
-	cache->ascent = 0;
 	g_slice_free(struct _vte_glyph_cache, cache);
 }
 
@@ -146,7 +125,7 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 	double dpi, size;
 	GList *iter;
 	FcPattern *pattern;
-	GArray *patterns;
+	GPtrArray *patterns;
 	FT_Face face;
 	gunichar double_wide_characters[] = {VTE_DRAW_DOUBLE_WIDE_CHARACTERS};
 
@@ -154,19 +133,19 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 	g_return_if_fail(fontdesc != NULL);
 
 	/* Convert the font description to a sorted set of patterns. */
-	patterns = g_array_new(TRUE, TRUE, sizeof(FcPattern*));
+	patterns = g_ptr_array_new();
 	if (!_vte_fc_patterns_from_pango_font_desc(widget, fontdesc,
 						   antialias,
 						   patterns,
 						   defaults_cb,
 						   defaults_data)) {
-		g_array_free(patterns, TRUE);
+		g_ptr_array_free(patterns, TRUE);
 		g_assert_not_reached();
 	}
 
 	/* Set the pattern list. */
 	if (cache->patterns != NULL) {
-		g_array_free(cache->patterns, TRUE);
+		g_ptr_array_free(cache->patterns, TRUE);
 	}
 	cache->patterns = patterns;
 
@@ -179,9 +158,8 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 	cache->faces = NULL;
 
 	/* Clear the glyph tree. */
-	g_tree_foreach(cache->cache, free_tree_value, NULL);
-	g_tree_destroy(cache->cache);
-	cache->cache = g_tree_new(_vte_direct_compare);
+	g_hash_table_destroy(cache->cache);
+	cache->cache = g_hash_table_new_full(NULL, NULL, NULL, _vte_cached_glyph_free);
 
 	/* Clear the load and render flags. */
 	cache->ft_load_flags = 0;
@@ -189,7 +167,7 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 
 	/* Open the all of the faces to which the patterns resolve. */
 	for (i = 0; i < cache->patterns->len; i++) {
-		pattern = g_array_index(cache->patterns, FcPattern*, i);
+		pattern = g_ptr_array_index(cache->patterns, i);
 		j = 0;
 		while (FcPatternGetString(pattern, FC_FILE, j,
 					  &facefile) == FcResultMatch) {
@@ -230,7 +208,7 @@ _vte_glyph_cache_set_font_description(GtkWidget *widget,
 	cache->ft_load_flags = 0;
 	cache->ft_render_flags = 0;
 	i = 0;
-	pattern = g_array_index(cache->patterns, FcPattern*, 0);
+	pattern = g_ptr_array_index(cache->patterns, 0);
 	/* Read and set the "use the autohinter", er, hint. */
 #if defined(FC_AUTOHINT) && defined(FT_LOAD_FORCE_AUTOHINT)
 	if (FcPatternGetBool(pattern, FC_AUTOHINT, 0, &i) == FcResultMatch) {
@@ -392,8 +370,8 @@ _vte_glyph_cache_has_char(struct _vte_glyph_cache *cache, gunichar c)
 	GList *iter;
 	gpointer p;
 
-	if ((p = g_tree_lookup(cache->cache, GINT_TO_POINTER(c))) != NULL) {
-		if (GPOINTER_TO_INT(p) == INVALID_GLYPH) {
+	if ((p = g_hash_table_lookup(cache->cache, GINT_TO_POINTER(c))) != NULL) {
+		if (p == INVALID_GLYPH) {
 			return FALSE;
 		}
 	}
@@ -408,14 +386,10 @@ _vte_glyph_cache_has_char(struct _vte_glyph_cache *cache, gunichar c)
 }
 
 static gunichar
-_vte_glyph_remap_char(struct _vte_glyph_cache *cache, gunichar origc)
+_vte_glyph_remap_char(gunichar origc)
 {
 	gunichar newc;
 
-	if (_vte_glyph_cache_has_char(cache, origc)) {
-		return origc;
-	}
-
 	switch (origc) {
 	case 0:			/* NUL */
 	case 0x00A0:		/* NO-BREAK SPACE */
@@ -434,11 +408,7 @@ _vte_glyph_remap_char(struct _vte_glyph_cache *cache, gunichar origc)
 		break;
 	}
 
-	if (_vte_glyph_cache_has_char(cache, newc)) {
-		return newc;
-	} else {
-		return origc;
-	}
+	return newc;
 }
 
 #define DEFAULT_BYTES_PER_PIXEL 3
@@ -496,8 +466,8 @@ _vte_glyph_get_uncached(struct _vte_glyph_cache *cache, gunichar c)
 
 	/* Bail if we weren't able to load the glyph. */
 	if (face == NULL) {
-		g_tree_insert(cache->cache, GINT_TO_POINTER(c),
-			      GINT_TO_POINTER(INVALID_GLYPH));
+		g_hash_table_insert(cache->cache, GINT_TO_POINTER(c),
+			      INVALID_GLYPH);
 		return NULL;
 	}
 
@@ -512,9 +482,6 @@ _vte_glyph_get_uncached(struct _vte_glyph_cache *cache, gunichar c)
 			  face->glyph->bitmap_top, 0);
 	glyph->bytes_per_pixel = DEFAULT_BYTES_PER_PIXEL;
 
-	memset(glyph->bytes, 0,
-	       glyph->width * glyph->height * DEFAULT_BYTES_PER_PIXEL);
-
 	for (y = 0; y < face->glyph->bitmap.rows; y++)
 	for (x = 0; x < face->glyph->bitmap.width; x++) {
 		ooffset = (y * glyph->width + x) * DEFAULT_BYTES_PER_PIXEL;
@@ -607,8 +574,8 @@ _vte_glyph_get(struct _vte_glyph_cache *cache, gunichar c)
 	g_return_val_if_fail(cache != NULL, NULL);
 
 	/* See if we already have a glyph for this character. */
-	if ((p = g_tree_lookup(cache->cache, GINT_TO_POINTER(c))) != NULL) {
-		if (GPOINTER_TO_INT(p) == INVALID_GLYPH) {
+	if ((p = g_hash_table_lookup(cache->cache, GINT_TO_POINTER(c))) != NULL) {
+		if (p == INVALID_GLYPH) {
 			return NULL;
 		} else {
 			return p;
@@ -620,13 +587,13 @@ _vte_glyph_get(struct _vte_glyph_cache *cache, gunichar c)
 
 	/* Bail if we weren't able to load the glyph. */
 	if (glyph == NULL) {
-		g_tree_insert(cache->cache, GINT_TO_POINTER(c),
-			      GINT_TO_POINTER(INVALID_GLYPH));
+		g_hash_table_insert(cache->cache, GINT_TO_POINTER(c),
+			      INVALID_GLYPH);
 		return NULL;
 	}
 
 	/* Cache it. */
-	g_tree_insert(cache->cache, GINT_TO_POINTER(c), glyph);
+	g_hash_table_insert(cache->cache, GINT_TO_POINTER(c), glyph);
 
 	return glyph;
 }
@@ -639,7 +606,7 @@ _vte_glyph_draw(struct _vte_glyph_cache *cache,
 		struct _vte_rgb_buffer *buffer)
 {
 	const struct _vte_glyph *glyph;
-	gint col, row, ioffset, ooffset, icol, ocol, ecol;
+	gint col, row, erow, ioffset, ooffset, icol, ocol, ecol;
 	gint strikethrough, underline, underline2;
 	gint32 r, g, b, ar, ag, ab;
 	guchar *pixels;
@@ -647,9 +614,14 @@ _vte_glyph_draw(struct _vte_glyph_cache *cache,
 	if (cache == NULL) {
 		return;
 	}
-	glyph = _vte_glyph_get(cache, _vte_glyph_remap_char(cache, c));
+	glyph = _vte_glyph_get(cache, c);
 	if (glyph == NULL) {
-		return;
+		gunichar cc = _vte_glyph_remap_char(c);
+		if(cc != c)
+			glyph = _vte_glyph_get(cache, cc);
+		if (glyph == NULL) {
+			return;
+		}
 	}
 
 	if (x > buffer->width) {
@@ -679,21 +651,18 @@ _vte_glyph_draw(struct _vte_glyph_cache *cache,
 
 _vte_glyph_draw_loop:
 
+	erow = MIN(cache->height, glyph->skip + glyph->height);
+	erow = MIN(erow, buffer->height - y);
 	for (row = glyph->skip;
-	     row < MIN(cache->height, glyph->skip + glyph->height);
+	     row < erow;
 	     row++) {
-		if (row + y >= buffer->height) {
-			break;
-		}
 		ooffset = (y + row) * buffer->stride +
 			  ((x + ocol) * 3);
 		ioffset = (((row - glyph->skip) * glyph->width) + icol) *
 			  DEFAULT_BYTES_PER_PIXEL;
 		ecol = MIN(cache->width * columns, glyph->width);
+		ecol = MIN(ecol, buffer->width - x);
 		for (col = 0; col < ecol; col++) {
-			if (col + x >= buffer->width) {
-				break;
-			}
 			ar = glyph->bytes[ioffset + 0];
 			ag = glyph->bytes[ioffset + 1];
 			ab = glyph->bytes[ioffset + 2];
