@@ -75,32 +75,22 @@ _vte_pango_destroy(struct _vte_draw *draw)
 	struct _vte_pango_data *data;
 	data = (struct _vte_pango_data*) draw->impl_data;
 
-	data->scrollx = data->scrolly = 0;
-
-	if (GDK_IS_PIXMAP(data->pixmap)) {
-		g_object_unref(G_OBJECT(data->pixmap));
-		data->pixmap = NULL;
-		data->pixmapw = data->pixmaph = 0;
+	if (data->pixmap != NULL) {
+		g_object_unref(data->pixmap);
 	}
 	if (data->font != NULL) {
 		pango_font_description_free(data->font);
-		data->font = NULL;
 	}
-	if (PANGO_IS_LAYOUT(data->layout)) {
-		g_object_unref(G_OBJECT(data->layout));
-		data->layout = NULL;
+	if (data->layout != NULL) {
+		g_object_unref(data->layout);
 	}
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
-		data->ctx = NULL;
+	if (data->ctx != NULL) {
+		g_object_unref(data->ctx);
 	}
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
-		data->gc = NULL;
+	if (data->gc != NULL) {
+		g_object_unref(data->gc);
 	}
 
-	memset(&data->color, 0, sizeof(data->color));
-
 	g_slice_free(struct _vte_pango_data, draw->impl_data);
 }
 
@@ -126,21 +116,21 @@ _vte_pango_start(struct _vte_draw *draw)
 
 	screen = gdk_drawable_get_screen(draw->widget->window);
 	ctx = gdk_pango_context_get_for_screen(screen);
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
+	if (data->ctx != NULL) {
+		g_object_unref(data->ctx);
 	}
 	data->ctx = ctx;
 
-	if (PANGO_IS_LAYOUT(data->layout)) {
-		g_object_unref(G_OBJECT(data->layout));
+	if (data->layout != NULL) {
+		g_object_unref(data->layout);
 	}
 	data->layout = pango_layout_new(data->ctx);
-	if (data->font) {
+	if (data->font != NULL) {
 		pango_layout_set_font_description(data->layout, data->font);
 	}
 
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
+	if (data->gc != NULL) {
+		g_object_unref(data->gc);
 	}
 	data->gc = gdk_gc_new(draw->widget->window);
 
@@ -154,18 +144,18 @@ _vte_pango_end(struct _vte_draw *draw)
 	struct _vte_pango_data *data;
 	data = (struct _vte_pango_data*) draw->impl_data;
 
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
+	if (data->gc != NULL) {
+		g_object_unref(data->gc);
 	}
 	data->gc = NULL;
 
-	if (PANGO_IS_LAYOUT(data->layout)) {
-		g_object_unref(G_OBJECT(data->layout));
+	if (data->layout != NULL) {
+		g_object_unref(data->layout);
 	}
 	data->layout = NULL;
 
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
+	if (data->ctx != NULL) {
+		g_object_unref(data->ctx);
 	}
 	data->ctx = NULL;
 }
@@ -197,8 +187,8 @@ _vte_pango_set_background_image(struct _vte_draw *draw,
 				   type, pixbuf, file,
 				   color, saturation,
 				   _vte_draw_get_colormap(draw, TRUE));
-	if (GDK_IS_PIXMAP(data->pixmap)) {
-		g_object_unref(G_OBJECT(data->pixmap));
+	if (data->pixmap != NULL) {
+		g_object_unref(data->pixmap);
 	}
 	data->pixmap = NULL;
 	data->pixmapw = data->pixmaph = 0;
@@ -326,13 +316,13 @@ _vte_pango_set_text_font(struct _vte_draw *draw,
 
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_MISC)) {
-		fprintf(stderr, "VtePango font metrics = %dx%d (%d).\n",
+		g_printerr("VtePango font metrics = %dx%d (%d).\n",
 			draw->width, draw->height, draw->ascent);
 	}
 #endif
 
-	g_object_unref(G_OBJECT(layout));
-	g_object_unref(G_OBJECT(ctx));
+	g_object_unref(layout);
+	g_object_unref(ctx);
 }
 
 static int
@@ -449,8 +439,8 @@ _vte_pango_set_scroll(struct _vte_draw *draw, gint x, gint y)
 	data->scrolly = y;
 }
 
-struct _vte_draw_impl _vte_draw_pango = {
-	"VtePango", "VTE_USE_PANGO",
+const struct _vte_draw_impl _vte_draw_pango = {
+	"Pango",
 	_vte_pango_check,
 	_vte_pango_create,
 	_vte_pango_destroy,
