@@ -16,7 +16,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: vtepangox.c,v 1.12 2003/05/28 21:53:59 nalin Exp $"
 
 #include "../config.h"
 
@@ -61,7 +60,7 @@ _vte_pango_x_create(struct _vte_draw *draw, GtkWidget *widget)
 {
 	struct _vte_pango_x_data *data;
 
-	draw->impl_data = g_malloc(sizeof(struct _vte_pango_x_data));
+	draw->impl_data = g_slice_new(struct _vte_pango_x_data);
 	data = (struct _vte_pango_x_data*) draw->impl_data;
 
 	data->color.red = 0;
@@ -82,33 +81,23 @@ _vte_pango_x_destroy(struct _vte_draw *draw)
 	struct _vte_pango_x_data *data;
 	data = (struct _vte_pango_x_data*) draw->impl_data;
 
-	data->scrollx = data->scrolly = 0;
-
-	if (GDK_IS_PIXMAP(data->pixmap)) {
-		g_object_unref(G_OBJECT(data->pixmap));
-		data->pixmap = NULL;
-		data->pixmapw = data->pixmaph = 0;
+	if (data->pixmap != NULL) {
+		g_object_unref(data->pixmap);
 	}
 	if (data->font != NULL) {
 		pango_font_description_free(data->font);
-		data->font = NULL;
 	}
-	if (PANGO_IS_LAYOUT(data->layout)) {
-		g_object_unref(G_OBJECT(data->layout));
-		data->layout = NULL;
+	if (data->layout != NULL) {
+		g_object_unref(data->layout);
 	}
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
-		data->gc = NULL;
+	if (data->gc != NULL) {
+		g_object_unref(data->gc);
 	}
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
-		data->ctx = NULL;
+	if (data->ctx != NULL) {
+		g_object_unref(data->ctx);
 	}
 
-	memset(&data->color, 0, sizeof(data->color));
-
-	g_free(draw->impl_data);
+	g_slice_free(struct _vte_pango_x_data, draw->impl_data);
 }
 
 static GdkVisual *
@@ -134,22 +123,22 @@ _vte_pango_x_start(struct _vte_draw *draw)
 	data = (struct _vte_pango_x_data*) draw->impl_data;
 
 	display = gdk_x11_drawable_get_xdisplay(draw->widget->window);
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
+	if (data->ctx != NULL) {
+		g_object_unref(data->ctx);
 	}
 	data->ctx = pango_x_get_context(display);
 
-	if (PANGO_IS_LAYOUT(data->layout)) {
-		g_object_unref(G_OBJECT(data->layout));
+	if (data->layout != NULL) {
+		g_object_unref(data->layout);
 	}
 	data->layout = pango_layout_new(data->ctx);
 
-	if (data->font) {
+	if (data->font != NULL) {
 		pango_layout_set_font_description(data->layout, data->font);
 	}
 
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
+	if (data->gc != NULL) {
+		g_object_unref(data->gc);
 	}
 	data->gc = gdk_gc_new(draw->widget->window);
 
@@ -173,24 +162,24 @@ _vte_pango_x_end(struct _vte_draw *draw)
 	data->x_offs = 0;
 	data->y_offs = 0;
 
-	if (PANGO_IS_LAYOUT(data->layout)) {
-		g_object_unref(G_OBJECT(data->layout));
+	if (data->layout != NULL) {
+		g_object_unref(data->layout);
 	}
 	data->layout = NULL;
 
-	if (GDK_IS_GC(data->gc)) {
-		g_object_unref(G_OBJECT(data->gc));
+	if (data->gc != NULL) {
+		g_object_unref(data->gc);
 	}
 	data->gc = NULL;
 
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
+	if (data->ctx != NULL) {
+		g_object_unref(data->ctx);
 	}
 	data->ctx = NULL;
 }
 
 static void
-_vte_pango_x_set_background_color(struct _vte_draw *draw, GdkColor *color)
+_vte_pango_x_set_background_color(struct _vte_draw *draw, GdkColor *color, guint16 opacity)
 {
 	struct _vte_pango_x_data *data;
 	data = (struct _vte_pango_x_data*) draw->impl_data;
@@ -207,17 +196,21 @@ _vte_pango_x_set_background_image(struct _vte_draw *draw,
 {
 	GdkPixmap *pixmap;
 	struct _vte_pango_x_data *data;
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (draw->widget);
 
 	data = (struct _vte_pango_x_data*) draw->impl_data;
-	pixmap = vte_bg_get_pixmap(vte_bg_get(), type, pixbuf, file,
+	pixmap = vte_bg_get_pixmap(vte_bg_get_for_screen(screen),
+				   type, pixbuf, file,
 				   color, saturation,
 				   _vte_draw_get_colormap(draw, TRUE));
-	if (data->pixmap) {
-		g_object_unref(G_OBJECT(data->pixmap));
+	if (data->pixmap != NULL) {
+		g_object_unref(data->pixmap);
 	}
 	data->pixmap = NULL;
 	data->pixmapw = data->pixmaph = 0;
-	if (pixmap) {
+	if (pixmap != NULL) {
 		data->pixmap = pixmap;
 		gdk_drawable_get_size(pixmap, &data->pixmapw, &data->pixmaph);
 	}
@@ -271,7 +264,8 @@ _vte_pango_x_clear(struct _vte_draw *draw,
 
 static void
 _vte_pango_x_set_text_font(struct _vte_draw *draw,
-			   const PangoFontDescription *fontdesc)
+			   const PangoFontDescription *fontdesc,
+			   VteTerminalAntiAlias antialias)
 {
 	PangoContext *ctx;
 	Display *display;
@@ -286,13 +280,9 @@ _vte_pango_x_set_text_font(struct _vte_draw *draw,
 
 	data = (struct _vte_pango_x_data*) draw->impl_data;
 
-#if GTK_CHECK_VERSION(2,2,0)
 	display = gdk_x11_display_get_xdisplay(gtk_widget_get_display(draw->widget));
-#else
-	display = gdk_display;
-#endif
-	if (PANGO_IS_CONTEXT(data->ctx)) {
-		g_object_unref(G_OBJECT(data->ctx));
+	if (data->ctx != NULL) {
+		g_object_unref(data->ctx);
 	}
 	ctx = pango_x_get_context(display);
 
@@ -317,7 +307,7 @@ _vte_pango_x_set_text_font(struct _vte_draw *draw,
 	pango_layout_iter_free(iter);
 
 	/* Estimate for CJK characters. */
-	full_string = g_string_new("");
+	full_string = g_string_new(NULL);
 	for (i = 0; i < G_N_ELEMENTS(full_codepoints); i++) {
 		g_string_append_unichar(full_string, full_codepoints[i]);
 	}
@@ -343,12 +333,12 @@ _vte_pango_x_set_text_font(struct _vte_draw *draw,
 
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_MISC)) {
-		fprintf(stderr, "VtePangoX font metrics = %dx%d (%d).\n",
+		g_printerr("VtePangoX font metrics = %dx%d (%d).\n",
 			draw->width, draw->height, draw->ascent);
 	}
 #endif
-	g_object_unref(G_OBJECT(layout));
-	g_object_unref(G_OBJECT(ctx));
+	g_object_unref(layout);
+	g_object_unref(ctx);
 }
 
 static int
@@ -369,6 +359,12 @@ _vte_pango_x_get_text_ascent(struct _vte_draw *draw)
 	return draw->ascent;
 }
 
+static int
+_vte_pango_x_get_char_width(struct _vte_draw *draw, gunichar c, int columns)
+{
+	return _vte_pango_x_get_text_width(draw) * columns;
+}
+
 static gboolean
 _vte_pango_x_get_using_fontconfig(struct _vte_draw *draw)
 {
@@ -464,8 +460,8 @@ _vte_pango_x_set_scroll(struct _vte_draw *draw, gint x, gint y)
 	data->scrolly = y;
 }
 
-struct _vte_draw_impl _vte_draw_pango_x = {
-	"VtePangoX", "VTE_USE_PANGOX",
+const struct _vte_draw_impl _vte_draw_pango_x = {
+	"PangoX",
 	_vte_pango_x_check,
 	_vte_pango_x_create,
 	_vte_pango_x_destroy,
@@ -481,6 +477,7 @@ struct _vte_draw_impl _vte_draw_pango_x = {
 	_vte_pango_x_get_text_width,
 	_vte_pango_x_get_text_height,
 	_vte_pango_x_get_text_ascent,
+	_vte_pango_x_get_char_width,
 	_vte_pango_x_get_using_fontconfig,
 	_vte_pango_x_draw_text,
 	_vte_pango_x_draw_char,
