@@ -43,8 +43,8 @@
 
 struct _vte_xft_font {
 	GdkDisplay *display;
-	GArray *patterns;
-	GArray *fonts;
+	GPtrArray *patterns;
+	GPtrArray *fonts;
 	VteTree *fontmap;
 	VteTree *widths;
 };
@@ -94,19 +94,19 @@ _vte_xft_font_open(GtkWidget *widget, const PangoFontDescription *fontdesc,
 		   VteTerminalAntiAlias antialias)
 {
 	struct _vte_xft_font *font;
-	GArray *patterns;
+	GPtrArray *patterns;
 
-	patterns = g_array_new(TRUE, TRUE, sizeof(FcPattern*));
+	patterns = g_ptr_array_new();
 	if (!_vte_fc_patterns_from_pango_font_desc(widget, fontdesc, antialias,
 						   patterns, NULL, NULL)) {
-		g_array_free(patterns, TRUE);
+		g_ptr_array_free(patterns, TRUE);
 		return NULL;
 	}
 
-	font = g_slice_new0(struct _vte_xft_font);
+	font = g_slice_new(struct _vte_xft_font);
 	font->display = gtk_widget_get_display(widget);
 	font->patterns = patterns;
-	font->fonts = g_array_new(TRUE, TRUE, sizeof(XftFont*));
+	font->fonts = g_ptr_array_new();
 	font->fontmap = _vte_tree_new(_vte_xft_direct_compare);
 	font->widths = _vte_tree_new(_vte_xft_direct_compare);
 
@@ -123,30 +123,26 @@ _vte_xft_font_close(struct _vte_xft_font *font)
 	int i;
 
 	for (i = 0; i < font->patterns->len; i++) {
-		pattern = g_array_index(font->patterns, FcPattern*, i);
+		pattern = g_ptr_array_index(font->patterns, i);
 		if (pattern == NULL) {
 			continue;
 		}
 		FcPatternDestroy(pattern);
 	}
-	g_array_free(font->patterns, TRUE);
-	font->patterns = NULL;
+	g_ptr_array_free(font->patterns, TRUE);
 
 	gdisplay = gdk_display_get_default();
 	display = gdk_x11_display_get_xdisplay(gdisplay);
 	for (i = 0; i < font->fonts->len; i++) {
-		ftfont = g_array_index(font->fonts, XftFont*, i);
+		ftfont = g_ptr_array_index(font->fonts, i);
 		if (ftfont != NULL) {
 			XftFontClose(display, ftfont);
 		}
 	}
-	g_array_free(font->fonts, TRUE);
-	font->fonts = NULL;
+	g_ptr_array_free(font->fonts, TRUE);
 
 	_vte_tree_destroy(font->fontmap);
-	font->fontmap = NULL;
 	_vte_tree_destroy(font->widths);
-	font->widths = NULL;
 
 	g_slice_free(struct _vte_xft_font, font);
 }
@@ -156,7 +152,6 @@ _vte_xft_font_for_char(struct _vte_xft_font *font, gunichar c)
 {
 	int i;
 	XftFont *ftfont;
-	FcPattern **patternp;
 	GdkDisplay *gdisplay;
 	Display *display;
 	gpointer p = GINT_TO_POINTER(c);
@@ -168,17 +163,14 @@ _vte_xft_font_for_char(struct _vte_xft_font *font, gunichar c)
 		/* Checked before, no luck. */
 		case -FONT_INDEX_FUDGE:
 			if (font->fonts->len > 0) {
-				return g_array_index(font->fonts,
-						     XftFont *,
-						     0);
+				return g_ptr_array_index(font->fonts, 0);
 			} else {
 				/* What to do? */
 				g_assert_not_reached();
 			}
 		/* Matched before. */
 		default:
-			return g_array_index(font->fonts,
-					     XftFont *,
+			return g_ptr_array_index(font->fonts,
 					     i - FONT_INDEX_FUDGE);
 		}
 	}
@@ -187,7 +179,7 @@ _vte_xft_font_for_char(struct _vte_xft_font *font, gunichar c)
 	gdisplay = gdk_display_get_default();
 	display = gdk_x11_display_get_xdisplay(gdisplay);
 	for (i = 0; i < font->fonts->len; i++) {
-		ftfont = g_array_index(font->fonts, XftFont *, i);
+		ftfont = g_ptr_array_index(font->fonts, i);
 		if ((ftfont != NULL) &&
 		    (_vte_xft_char_exists(font, ftfont, c))) {
 			break;
@@ -199,7 +191,7 @@ _vte_xft_font_for_char(struct _vte_xft_font *font, gunichar c)
 		_vte_tree_insert(font->fontmap,
 			      p,
 			      GINT_TO_POINTER(i + FONT_INDEX_FUDGE));
-		ftfont = g_array_index(font->fonts, XftFont *, i);
+		ftfont = g_ptr_array_index(font->fonts, i);
 		if (ftfont != NULL) {
 			return ftfont;
 		} else {
@@ -209,13 +201,13 @@ _vte_xft_font_for_char(struct _vte_xft_font *font, gunichar c)
 
 	/* Look the character up in other fonts. */
 	for (i = font->fonts->len; i < font->patterns->len; i++) {
-		patternp = &g_array_index(font->patterns, FcPattern *, i);
-		ftfont = XftFontOpenPattern(display, *patternp);
+		ftfont = XftFontOpenPattern(display,
+			       	g_ptr_array_index(font->patterns, i));
 		/* If the font was opened, it owns the pattern. */
 		if (ftfont != NULL) {
-			*patternp = NULL;
+			g_ptr_array_index(font->patterns, i) = NULL;
 		}
-		g_array_append_val(font->fonts, ftfont);
+		g_ptr_array_add(font->fonts, ftfont);
 		if ((ftfont != NULL) &&
 		    (_vte_xft_char_exists(font, ftfont, c))) {
 			break;
@@ -228,9 +220,7 @@ _vte_xft_font_for_char(struct _vte_xft_font *font, gunichar c)
 			      p,
 			      GINT_TO_POINTER(-FONT_INDEX_FUDGE));
 		if (font->fonts->len > 0) {
-			return g_array_index(font->fonts,
-					     XftFont *,
-					     0);
+			return g_ptr_array_index(font->fonts, 0);
 		} else {
 			/* What to do? */
 			g_assert_not_reached();
@@ -243,7 +233,7 @@ _vte_xft_font_for_char(struct _vte_xft_font *font, gunichar c)
 
 	/* Return the match. */
 	if (i < font->fonts->len) {
-		return g_array_index(font->fonts, XftFont *, i);
+		return g_ptr_array_index(font->fonts, i);
 	} else {
 		return NULL;
 	}
@@ -314,30 +304,14 @@ _vte_xft_destroy(struct _vte_draw *draw)
 	data = (struct _vte_xft_data*) draw->impl_data;
 	if (data->font != NULL) {
 		_vte_xft_font_close(data->font);
-		data->font = NULL;
 	}
-	data->display = NULL;
-	data->drawable = -1;
-	data->x_offs = data->y_offs = 0;
-	data->visual = NULL;
 	data->colormap = -1;
 	if (data->draw != NULL) {
 		XftDrawDestroy(data->draw);
-		data->draw = NULL;
-		data->display = NULL;
-		data->drawable = -1;
-		data->visual = NULL;
-		data->colormap = -1;
 	}
 	if (data->gc != NULL) {
 		XFreeGC(data->display, data->gc);
-		data->gc = NULL;
 	}
-	memset(&data->color, 0, sizeof(data->color));
-	data->pixmap = NULL;
-	data->xpixmap = -1;
-	data->pixmapw = data->pixmaph = -1;
-	data->scrollx = data->scrolly = 0;
 	g_slice_free(struct _vte_xft_data, data);
 }
 
@@ -434,11 +408,11 @@ _vte_xft_set_background_image(struct _vte_draw *draw,
 	pixmap = vte_bg_get_pixmap(vte_bg_get_for_screen(screen), type,
 				   pixbuf, file, color, saturation,
 				   _vte_draw_get_colormap(draw, TRUE));
-	if (GDK_IS_PIXMAP(data->pixmap)) {
-		g_object_unref(G_OBJECT(data->pixmap));
+	if (data->pixmap != NULL) {
+		g_object_unref(data->pixmap);
 	}
 	data->pixmap = NULL;
-	if (GDK_IS_PIXMAP(pixmap)) {
+	if (pixmap != NULL) {
 		data->pixmap = pixmap;
 		data->xpixmap = gdk_x11_drawable_get_xid(pixmap);
 		gdk_drawable_get_size(pixmap, &data->pixmapw, &data->pixmaph);
@@ -456,7 +430,7 @@ _vte_xft_clear(struct _vte_draw *draw,
 
 	data = (struct _vte_xft_data*) draw->impl_data;
 
-	if (!GDK_IS_PIXMAP(data->pixmap) ||
+	if (data->pixmap == NULL ||
 	    (data->pixmapw <= 0) ||
 	    (data->pixmaph <= 0)) {
 		rcolor.red = data->color.red * data->opacity / 0xffff;
@@ -572,7 +546,7 @@ _vte_xft_set_text_font(struct _vte_draw *draw,
 
 #ifdef VTE_DEBUG
 	if (_vte_debug_on(VTE_DEBUG_MISC)) {
-		fprintf(stderr, "VteXft font metrics = %dx%d (%d).\n",
+		g_printerr("VteXft font metrics = %dx%d (%d).\n",
 			draw->width, draw->height, draw->ascent);
 	}
 #endif
@@ -788,8 +762,8 @@ _vte_xft_set_scroll(struct _vte_draw *draw, gint x, gint y)
 	data->scrolly = y;
 }
 
-struct _vte_draw_impl _vte_draw_xft = {
-	"VteXft", "VTE_USE_XFT",
+const struct _vte_draw_impl _vte_draw_xft = {
+	"Xft",
 	_vte_xft_check,
 	_vte_xft_create,
 	_vte_xft_destroy,
